@@ -61,7 +61,7 @@ void update_grace_times(struct dquot *q)
 /*
  * Collect the requested quota information.
  */
-struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
+struct dquot *getprivs(qid_t id, struct quota_handle **handles, int ignore_noquota)
 {
 	struct dquot *q, *qtail = NULL, *qhead = NULL;
 	int i;
@@ -80,7 +80,7 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 				if (euid != id && euid != 0) {
 					uid2user(id, name);
 					errstr(_("%s (uid %d): Permission denied\n"), name, id);
-					return (struct dquot *)NULL;
+					goto out_err;
 				}
 				break;
 			case GRPQUOTA:
@@ -92,7 +92,7 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 					if (!gidsetp) {
 						gid2group(id, name);
 						errstr(_("%s (gid %d): gid set allocation (%d): %s\n"), name, id, ngroups, strerror(errno));
-						return (struct dquot *)NULL;
+						goto out_err;
 					}
 				}
 				else
@@ -103,7 +103,7 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 						free(gidsetp);
 					gid2group(id, name);
 					errstr(_("%s (gid %d): error while trying getgroups(): %s\n"), name, id, strerror(errno));
-					return (struct dquot *)NULL;
+					goto out_err;
 				}
 
 				for (j = 0; j < ngroups; j++)
@@ -115,7 +115,7 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 					gid2group(id, name);
 					errstr(_("%s (gid %d): Permission denied\n"),
 						name, id);
-					return (struct dquot *)NULL;
+					goto out_err;
 				}
 				break;
 			default:
@@ -124,15 +124,25 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 #endif
 
 		if (!(q = handles[i]->qh_ops->read_dquot(handles[i], id))) {
-			/* If rpc.rquotad is not running filesystem might be just without quotas... */
-			if (errno != ENOENT && (errno != ECONNREFUSED || !quiet)) {
-				int olderrno = errno;
+			char *estr;
 
-				id2name(id, handles[i]->qh_type, name);
-				errstr(_("error while getting quota from %s for %s (id %u): %s\n"),
-					handles[i]->qh_quotadev, name, id, strerror(olderrno));
+			/* If rpc.rquotad is not running, filesystem might be just without quotas... */
+			if (ignore_noquota && (errno == ENOENT || errno == ECONNREFUSED))
+				continue;
+
+			if (errno == ECONNREFUSED) {
+				estr = _("Cannot connect to RPC quota service");
+			} else if (errno == ENOENT) {
+				estr = _("Quota not enabled");
+			} else {
+				estr = strerror(errno);
 			}
-			continue;
+			id2name(id, handles[i]->qh_type, name);
+			errstr(_("error while getting quota from %s for %s (id %u): %s\n"),
+				handles[i]->qh_quotadev, name, id, estr);
+out_err:
+			freeprivs(qhead);
+			return NULL;
 		}
 		if (qhead == NULL)
 			qhead = q;
